local satcheler = LibStub("AceAddon-3.0"):NewAddon("Satcheler", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("Satcheler", true)
local satchelerFrame = CreateFrame("Frame")
--local tank, healer, damage, queueId
local dialogShown = false

StaticPopupDialogs["SATCHELER_JOIN_LFD"] = {
	text = "Do you want to greet the world today?",
	button1 = "Yes",
	button2 = "No",
	OnAccept = function(self)
		satcheler:joinLFD(self.role, self.queueID)
		satcheler.db.profile.delay = true
	end,
	OnCancel = function()
		satcheler.db.profile.delay = true
	end,
	OnShow = function()
		dialogShown = true
	end,
	OnHide = function()
		dialogShown = false
	end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	preferredIndex = 3,  -- avoid some UI taint, see http://www.wowace.com/announcements/how-to-avoid-some-ui-taint/
}

local defaults = {
	profile = {
		selectedRoles = {
			forTanks = false,
			forHealers = false,
			forDamage = false,
		},
		satchelRoles = {
			forTanks = false,
			forHealers = false,
			forDamage = false,
		},
		delay = false,
	}
}

function satcheler:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("YurrCombatLogDB", defaults, true)
end

function satcheler:OnEnable()
	satchelerFrame:SetScript("OnUpdate", satchelerFrame.OnUpdate)
end

local delay = 600
local lastupdate = 0
function satchelerFrame.OnUpdate(_, elapsed)
	lastupdate = lastupdate + elapsed
	
	if ((satcheler.db.profile.delay) and (lastupdate < delay)) or (GetNumRaidMembers() > 1) or (dialogShown) then
		return
	end
	
	satcheler:checkDungeonList()
	satcheler.db.profile.delay = false
	lastupdate = 0
end

function satcheler:detectShortage(dungeonID, name)
	queueId = dungeonID
	local roles = {}
	--for i = 1, LFG_ROLE_NUM_SHORTAGE_TYPES do -- for the different shortage levels
		local eligible, tank, healer, damage = GetLFGRoleShortageRewards(dungeonID, 1)
		local _, _, _, _, _, numRewards = GetLFGDungeonRewards(dungeonID)
		
		table.insert(roles, tank)
		table.insert(roles, healer)
		table.insert(roles, damage)
		
		if (tank or healer or damage) then
			local role = ""
			for i = 1, 3 do
				if roles[i] then
					if roles[1] then
						role = "Tank"
						tank = true
					end
					if roles[2] then
						role = role .. "Healer"
						healer = true
					end
					if roles[3] then
						role = role .. "Damage"
						damage = true
					end
					if roles[i+1] then
						role = role .. ", "
					end
				end
			end
			
			--self:satchelAvailable(dungeonID, name, role)
		--end
		
		for rewardIndex = 1, numRewards do
			local name = GetLFGDungeonRewardInfo(dungeonID, rewardIndex)
			if name == L["Satchel of Exotic Mysteries"] then
				self:satchelAvailable(dungeonID, name, role)
			end
		end
	end
end

function satcheler:checkDungeonList()
	RefreshLFGList() 
	for i = 1, GetNumRandomDungeons() do
	  local id, name = GetLFGRandomDungeonInfo(i)
	  self:detectShortage(id, name)
	end
end

function satcheler:satchelAvailable(dungeonID, name, role)
	--local text = "Satchel available for: " .. name .. "!" .. " Role: " .. role
	local text = format("Satchel available for : %s! Role: %s", name, role)
	StaticPopupDialogs["SATCHELER_JOIN_LFD"].text = text
	local dialog = StaticPopup_Show ("SATCHELER_JOIN_LFD")
	if (dialog) then
		dialog.role = role
		dialog.queueID = dungeonID
	end
	
	ChatFrame1:AddMessage(text)
end

function satcheler:joinLFD(role, queueID)
	local tank = false
	local healer = false
	local damage = false
	
	ChatFrame1:AddMessage("Role: " .. role .. "queueID:" .. queueID)
	
	local leader = GetLFGRoles()
	if (role == "Tank") then
		tank = true
	elseif (role == "Healer") then
		healer = true
	elseif (role == "Damage") then
		damage = true
	end
	ChatFrame1:AddMessage(tostring(tank) .. tostring(healer) .. tostring(damage))
	SetLFGRoles(leader, tank, healer, damage)
	
	ShowUIPanel(LFDParentFrame)
	local queueId = LFDQueueFrame.type
	SetLFGDungeon(queueId)
	JoinLFG()
end